package fontify;
#|########################################################################
# ftagshtml: transforms fortran and C source into HTML or PostScript or PDF.
#    Copyright (C) 2000 Olivier Cessenat.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#|########################################################################
sub start_liensindex {
    return ("<FONT CLASS=\"liensindex\">") ;
}
#|########################################################################
sub start_string {
    return ("$START_STRING") ;
}
#|########################################################################
sub end_string {
    return ("$END_STRING") ;
}
#|########################################################################
sub start_test {
    return ("$START_TEST") ;
}
#|########################################################################
sub end_test {
    return ("$END_TEST") ;
}
#|########################################################################
sub start_read {
    return ("$START_READ") ;
}
#|########################################################################
sub end_read {
    return ("$END_READ") ;
}
#|########################################################################
sub start_loop {
    return ("$START_LOOP") ;
}
#|########################################################################
sub end_loop {
    return ("$END_LOOP") ;
}
#|########################################################################
sub start_header {
    return ("$START_HEADER") ;
}
#|########################################################################
sub end_header {
    return ("$END_HEADER") ;
}
#|########################################################################
sub start_declare {
    return ("$START_DECLARE") ;
}
#|########################################################################
sub end_declare {
    return ("$END_DECLARE") ;
}
#|########################################################################
sub start_branchement {
    return ("$START_BRANCHEMENT") ;
}
#|########################################################################
sub end_branchement {
    return ("$END_BRANCHEMENT") ;
}
#|########################################################################
sub start_verbatim {
    return ("$START_VERBATIM") ;
}
#|########################################################################
sub end_verbatim {
    return ("$END_VERBATIM") ;
}
#|########################################################################
sub start_code {
    return ("$START_CODE") ;
}
#|########################################################################
sub end_code {
    return ("$END_CODE") ;
}
#|########################################################################
sub start_comment {
    return ("$START_COMMENT") ;
}
#|########################################################################
sub end_comment {
    return ("$END_COMMENT") ;
}
#|########################################################################
sub Fortran_Fontify {
    local($ligne)=@_;
    if ($ligne =~ /\s+(common|record|data|equivalence|implicit|parameter|pointer)\b/i) {
	$r = &fontify::start_read."$1".&fontify::end_read;
	$ligne =~ s/$1/$r/;
    } elsif ($ligne =~ /\s+(character|complex|dimension|integer|logical|real)\b/i) {
	$r = &fontify::start_declare."$1".&fontify::end_declare ;
	$ligne =~ s/$1/$r/;
    } elsif ($ligne =~ /\b(do|enddo|end\s+do)\b/i) {
	$r=&fontify::start_loop."$1".&fontify::end_loop ;
	$ligne =~ s/$1/$r/;
    } elsif ($ligne =~ /([\d\s]+)\b(if|elseif|endif|else|end\s+if|else\s+if)\b/i) {
	$a=$1;
	$r = &fontify::start_test."$2".&fontify::end_test;
	$ligne =~ s/$a$2/$a$r/;
    }
    return ($ligne);
} ;
#|########################################################################
sub F90_Fontify {
    local($ligne)=@_;
    if ($ligne =~ /\s+(common|record|data|equivalence|implicit|parameter|pointer)\b/i) {
	$r = &fontify::start_read."$1".&fontify::end_read;
	$ligne =~ s/$1/$r/;
    }
    elsif ($ligne =~ /\s+(character|complex|dimension|integer|logical|real|type)\b/i) {
	$r=&fontify::start_declare."$1".&fontify::end_declare ;
	$ligne =~ s/$1/$r/;
    }
    elsif ($ligne =~ /\b(do|enddo|end\s+do)\b/i) {
	$r=&fontify::start_loop."$1".&fontify::end_loop ;
	$ligne =~ s/$1/$r/;
    }
    elsif ($ligne =~ /([\d\s]*)\b(if|elseif|endif|else|end\s+if|else\s+if)\b/i) {
	$a=$1;
	$r = &fontify::start_test."$2".&fontify::end_test;
	$ligne =~ s/$a$2/$a$r/;
    }
    $ligne = searchfc::PRECode($ligne) ;
    return ($ligne);
} ;
#|#######################################################
sub ftagshtml_color {
    $Couleur_Item="Orange";
    $Couleur_Code="#660000"; # BROWN
    $Couleur_Verbatim="GREEN";
    $Couleur_Header="MAGENTA";
    $Couleur_LiensIndex="GREEN";
    $Couleur_Branchement="#6600EE"; # VIOLET
    $Couleur_Comment="BLACK";
    $Couleur_Utilisateur =0;
    if (defined($ENV{Couleur_Code})){
	$Couleur_Utilisateur =1;
	$Couleur_Code=$ENV{Couleur_Code};
    }
    if (defined($ENV{Couleur_Verbatim})){
	$Couleur_Utilisateur =1;
	$Couleur_Verbatim=$ENV{Couleur_Verbatim};
    }
    if (defined($ENV{Couleur_Item})){
	$Couleur_Utilisateur =1;
	$Couleur_Item=$ENV{Couleur_Item};
    }
    if (defined($ENV{Couleur_Header})){
	$Couleur_Utilisateur =1;
	$Couleur_Header=$ENV{Couleur_Header};
    }
    if (defined($ENV{Couleur_LiensIndex})){
	$Couleur_Utilisateur =1;
	$Couleur_LiensIndex=$ENV{Couleur_LiensIndex};
    }
    if (defined($ENV{Couleur_Branchement})){
	$Couleur_Utilisateur =1;
	$Couleur_Branchement=$ENV{Couleur_Branchement};
    }
    if (defined($ENV{Couleur_Comment})){
	$Couleur_Utilisateur =1;
	$Couleur_Comment=$ENV{Couleur_Comment};
    }
    unless ($Couleur_Utilisateur){
	if ($main::opt_v >= 1) {
	    print STDOUT "\tYou may define your private colours using environment variables:\n";
	    print STDOUT "\t\tCouleur_Code='$Couleur_Code'\n";
	    print STDOUT "\t\tCouleur_Header='$Couleur_Header'\n";
	    print STDOUT "\t\tCouleur_LiensIndex='$Couleur_LiensIndex'\n";
	    print STDOUT "\t\tCouleur_Branchement='$Couleur_Branchement'\n";
	    print STDOUT "\t\tCouleur_Verbatim='$Couleur_Verbatim'\n";
	    print STDOUT "\t\tCouleur_Item='$Couleur_Item'\n";
	    print STDOUT "\t\tCouleur_Comment='$Couleur_Comment'\n";
	}
    }
    $Couleur_Read='#FF7799';
    $Couleur_Declare='#FF9900';
    $Couleur_Loop='#33DD99';
    $Couleur_Test='#33DDDF';
    $Couleur_String=$Couleur_Comment ;
    if (!$main::opt_nocolor) {
	$Couleur_Utilisateur =0;
	if (defined($ENV{Couleur_Read})){
	    $Couleur_Utilisateur =1;
	    $Couleur_Read=$ENV{Couleur_Read};
	}
	if (defined($ENV{Couleur_Declare})){
	    $Couleur_Utilisateur =1;
	    $Couleur_Declare=$ENV{Couleur_Declare};
	}
	if (defined($ENV{Couleur_Loop})){
	    $Couleur_Utilisateur =1;
	    $Couleur_Loop=$ENV{Couleur_Loop};
	}
	if (defined($ENV{Couleur_Test})){
	    $Couleur_Utilisateur =1;
	    $Couleur_Test=$ENV{Couleur_Test};
	}
	if ($Couleur_Utilisateur){
	    print STDOUT "\tUsing Environment variables to fontify code:\n";
	    print STDOUT "\t\tCouleur_Read='$Couleur_Read'\n";
	    print STDOUT "\t\tCouleur_Declare='$Couleur_Declare'\n";
	    print STDOUT "\t\tCouleur_Loop='$Couleur_Loop'\n";
	    print STDOUT "\t\tCouleur_Test='$Couleur_Test'\n";
	    print STDOUT "\t\tCouleur_String='$Couleur_String'\n";
	}
    }
}
#|#######################################################
sub InitCodeComments {
    my($language) = @_ ;
    if ($language ne "void") {
	$INIT_CODE="";
	$TERMIN_CODE="";
	if ($language eq "f90") {
	    $START_CODE="";
	    $END_CODE="";
	} else {
	    $START_CODE="<PRE>";
	    $END_CODE="</PRE>";
	}
	if ($main::opt_nostyle) {
	    $START_CODE .= "<FONT COLOR=$Couleur_Code>";
	} else {
	    $START_CODE .= "<FONT CLASS=\"code\">";
	}
	$END_CODE ="</FONT>".$END_CODE ;
    }
    if ($main::opt_nostyle) {
	$START_HEADER = "<FONT COLOR=$Couleur_Header>";
    } else {
	$START_HEADER = "<FONT CLASS=\"header\">";
    }
    $END_HEADER = "</FONT>" ;
    if ($main::opt_nostyle) {
	$START_BRANCHEMENT = "<FONT COLOR=$Couleur_Branchement>";
    } else {
	$START_BRANCHEMENT = "<FONT CLASS=\"branchement\">";
    }
    $END_BRANCHEMENT = "</FONT>" ;
    if ($main::opt_nostyle) {
	$START_READ = "<FONT COLOR=$Couleur_Read>";
    } else {
	$START_READ = "<FONT CLASS=\"read\">";
    }
    $END_READ = "</FONT>" ;
    if ($main::opt_nostyle) {
	$START_VERBATIM = "<FONT COLOR=$Couleur_Verbatim>";
    } else {
	$START_VERBATIM = "<FONT CLASS=\"verbatim\">";
    }
    $END_VERBATIM = "</FONT>" ;
    if ($main::opt_nostyle) {
	$START_DECLARE = "<STRONG><FONT COLOR=$Couleur_Declare>";
    } else {
	$START_DECLARE = "<STRONG><FONT CLASS=\"declare\">";
    }
    $END_DECLARE = "</FONT></STRONG>" ;
    if ($main::opt_nostyle) {
	$START_TEST = "<I><FONT COLOR=$Couleur_Test>";
    } else {
	$START_TEST = "<I><FONT CLASS=\"test\">";
    }
    $END_TEST = "</FONT></I>" ;
    if ($main::opt_nostyle) {
	$START_LOOP = "<I><FONT COLOR=$Couleur_Loop>";
    } else {
	$START_LOOP = "<I><FONT CLASS=\"loop\">";
    }
    $END_LOOP = "</FONT></I>" ;
    if ($main::opt_nostyle) {
	$START_STRING = "<TT><I><FONT COLOR=$Couleur_String>";
    } else {
	$START_STRING = "<TT><I><FONT CLASS=\"string\">";
    }
    $END_STRING = "</FONT></I></TT>" ;
    if ($Full_Html) {
	$START_COMMENT="";
	$END_COMMENT="";
	$ligne_ini = "</FONT></PRE>".&fontify::start_liensindex."\n" ;
    } else {
	$START_COMMENT="<PRE>";
	$END_COMMENT="</PRE>";
	$ligne_ini = "</FONT></PRE>".&fontify::start_liensindex."\n" ;
    }
    if ($main::opt_nostyle) {
	$START_COMMENT .= "<FONT COLOR=$Couleur_Comment><b>";
    } else {
	$START_COMMENT .= "<FONT CLASS=\"comment\"><b>";
    }
    $END_COMMENT ="</b></FONT> ". $END_COMMENT ;
}
#|########################################################################
sub stylesheet {
    return("<LINK REL=STYLESHEET TYPE=\"text/css\" HREF=\"$main::Myname.css\">") ;
}
#|########################################################################
sub writesheet {
    if (open(CSS,">$main::DIRE/$main::Myname.css")) {
	print CSS '
FONT.code {color:'.$Couleur_Code.'; font-family: Arial, serif; background-color:#ddffff;}
FONT.comment {color:'.$Couleur_Comment.'; text-style:bold; background-color:#ffffdd; font-family: Gothic, roman}
FONT.header {color:'.$Couleur_Header.'; background-color:#ccffdd}
FONT.branchement {color:'.$Couleur_Branchement.'; background-color:#aaffaa}
FONT.verbatim {color:'.$Couleur_Verbatim.'; background-color:#ffaadd}
FONT.liensindex {color:'.$Couleur_LiensIndex.'; background-color:#ccbbee}
FONT.declare {color:'.$Couleur_Declare.'}
FONT.test {color:'.$Couleur_Test.'}
FONT.read {color:'.$Couleur_Read.'; text-style:underline}
FONT.loop {color:'.$Couleur_Loop.'; text-style:underline}
FONT.string {color:'.$Couleur_String.'; font-size:14pt; text-decoration:normal; font-style:small-caps; text-style:italic; font-family: Arial, serif; background-color:#ffffdd}
FONT.info {color:RED; text-style:normal; background-color:WHITE}
FONT.getin {color:RED; text-style:normal; background-color:GREEN}
' ;
    }
}
#|########################################################################
1;
