package searchfc;
#|########################################################################
# searchfc library: produces fortran 77, fortran 90 and C syntax analysis.
#    Copyright (C) 2000 Olivier Cessenat.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#|########################################################################
# Retourne si la ligne est un commentaire fortran
sub Fortran_Comment {
    local($ligne)=@_;
    if (( $ligne =~ /^[^\s]/i) && ( $ligne =~ /^[^\d]/i)) {
	return 1; # valeur true
    } else {
	$main::instruction = $ligne ;
	return 0; # valeur false
    }
}
#|########################################################################
# returns the instruction part of the line stripped off its strings
sub StripStringsF77 {
    my($ligne) = @_ ;
    while (($ligne) && ($ligne =~ /(.*?)([\'])(.*)/)) {
	$previous = $1 ;
	$symbol = $2 ;
	$ligne = $3 ;
    }
    return($ligne) ;
}
#|########################################################################
# returns the instruction part of the line stripped off its strings
# as well as the comment's part of the line.
# Tells whether contxt is still a string one.
# A further analysis of the instruction variable will tell whether
# the line is to be continued or not.
# This function can be used for includes analysis as well as for
# definitions'analysis (second and first stage of ftagshtml).
sub Fortran90_String_Comment {
# string=' means string context is '
# comment=1 means context is comment
    local($string,$ligne)=@_;
    local($instruction,$commentaire) = ("","") ;
    local($symbol,$previous) ;
    unless ( $ligne =~ /[\'\"\!]/ ) {
	if ("$string" eq "") {
	    $instruction .= $ligne ;
	}
	$ligne = "" ;
    }
# we must first strip off the strings of possible comment zones.
    while (($ligne) && ($ligne =~ /(.*?)([\'\"\!])(.*)/)) {
	$previous = $1 ;
	$symbol = $2 ;
	$ligne = $3 ;
	if ("$symbol" eq '!') {
	    if ("$string" eq "") {
# We meet a comment instruction that is not in a string context
		$instruction .= $previous ;
		$commentaire = $ligne ;
		$ligne = "";
		last ;
	    }
	} elsif ("$symbol" eq "$string") {
# means we are in string context and it is the end of it
	    $string="";
	} else {
	    if ("$string" eq "") {
# we were not in string context,
# but now we are using string symbol $symbol
# ... unless previous instruction was "include" !!!
		if ($previous =~ /include\s*$/i ) {
		    if ($ligne =~ /(.*?)$symbol(.*)/) {
# The instruction line will include the 'include filename' sentence
# stripped off from its string delimiter. We must remind this in any
# further includes'analysis.
			$instruction .= "$previous"." $1";
			$ligne = $2 ;
		    }
		} else {
		    $string = "$symbol" ;
		    $instruction .= $previous ;
		}
	    }
	}
    }
    $instruction .= $ligne if $ligne ;
    return("$string","$instruction","$commentaire");
}

#|#######################################################
sub F90LatexBuiltinProcess {
    local($ligne)=@_;
    $ligne=main::RefDeclare($ligne);
    $ligne=main::LatexProcess($ligne);
    $ligne=PREComment($ligne);
    return($ligne);
}
#|#######################################################
sub  PREComment{
# Protects the style of the comments in case no latex.
    local($ligne)=@_;
    $ligne =~ s/  /'&nbsp;&nbsp;'/geo unless $main::opt_latex ;
    return($ligne);
}
#|#######################################################
sub  PRECode{
# Protects the style of the comments in case no latex.
    local($ligne)=@_;
    $ligne =~ s/  /'&nbsp;&nbsp;'/geo ;
    return($ligne);
}
#|########################################################################
# returns the instruction part of the line stripped off its strings
# as well as the comment's part of the line.
# Tells whether contxt is still a string one.
# A further analysis of the instruction variable will tell whether
# the line is to be continued or not.
# This function can be used for includes analysis as well as for
# definitions'analysis (second and first stage of ftagshtml).
sub HTML_90_String_Comment {
# string=' means string context is '
# comment=1 means context is comment
    local($string,$comm,$ligne)=@_;
    local($instruction,$commentaire,$htmlligne) = ("","","") ;
    local($symbol,$previous) ;
    unless ( $ligne =~ /[\'\"\!]/ ) {
	if ("$string" eq "") {
	    $instruction .= $ligne ;
	    $htmlligne = &fontify::end_comment.&fontify::start_code if $comm ;
	    $htmlligne .= '<BR>'.fontify::F90_Fontify($ligne) ;
	    $comm = 0 ;
	}
	$ligne = "" ;
    }
    local($compteur)=0 ;
# we must first strip off the strings of possible comment zones.
    while (($ligne) && ($ligne =~ /(.*?)([\'\"\!])(.*)/)) {
	$previous = $1 ;
	$symbol = $2 ;
	$ligne = $3 ;
	$compteur++ ;
	if ("$symbol" eq '!') {
	    if ("$string" eq "") {
# We meet a comment instruction that is not in a string context
		$instruction .= $previous ;
		$commentaire = $ligne ;
		if (($previous)&&!($previous =~ /^\s+$/)) {
		    $htmlligne .= &fontify::end_comment.&fontify::start_code if $comm ;
		    if ($compteur==1) {
			$htmlligne .= '<BR>'  ;
		    }
		    $htmlligne .= fontify::F90_Fontify($previous) ;
		    $htmlligne .= &fontify::end_code.&fontify::start_comment."$symbol" ;
		} else {
		    if (($comm)&&($compteur==1)){
			$htmlligne .= '<BR>' unless $main::opt_latex ;
		    }
		    $htmlligne .= &fontify::end_code.&fontify::start_comment.PREComment("$previous")."$symbol" unless $comm ;
		    $htmlligne .= PREComment("$previous") if $comm ;
		}
		$htmlligne .= F90LatexBuiltinProcess($ligne) ;
		$commentaire = $ligne ;
		$ligne = "";
		$comm = 1 ;
		last ;
	    } else {
		$htmlligne .= fontify::F90_Fontify($previous) ;
	    }
	} else {
	    $htmlligne .= '<BR>' if ($compteur == 1) ;
	    $htmlligne .= &fontify::end_comment.&fontify::start_code if $comm ;
	    $comm = 0 ;
	    if ("$symbol" eq "$string") {
# means we are in string context and it is the end of it
		$htmlligne .= fontify::F90_Fontify($previous)."$string".&fontify::end_string ;
		$string="";
	    } elsif ("$string" eq "") {
# we were not in string context,
# but now we are using string symbol $symbol
# ... unless previous instruction was "include" !!!
		if ($previous =~ /include\s*$/i ) {
		    if ($ligne =~ /(.*?)$symbol(.*)/) {
# The instruction line will include the 'include filename' sentence
# stripped off from its string delimiter. We must remind this in any
# further includes' analysis.
			$instruction .= "$previous"." $1";
			$ligne = $2 ;
			$htmlligne .= "$previous"." $1";
		    } else {
			$htmlligne .= "$previous" ;
		    }
		} else {
		    $string = "$symbol" ;
		    $instruction .= $previous ;
		    $htmlligne .= &fontify::F90_Fontify($previous).&fontify::start_string."$symbol" ;
		}
	    }
	}
    }
    $htmlligne .= fontify::F90_Fontify($ligne) if $ligne ;
    $instruction .= $ligne if $ligne ;
    return("$string","$comm","$instruction","$commentaire","$htmlligne");
}

#|#######################################################
sub Find_Parenthesis {
    local($paren,$instruction)=@_;
    local($end_paren)=0;
    unless ($instruction) { return ($paren,0,0); }
    while ( $instruction =~ /(.*?)(\(|\))(.*)/ ) { # parenthese: complet
	if ($3) { $instruction = $3 ;} else { $instruction = 0; };
	if ($end_paren) { $end_paren .= $1.$2; } else { $end_paren = $1.$2; };
	$parenmoved=1;
	if ($2 eq ')') {
	    $paren=$paren-1;
	} elsif ($2 eq '(') {
	    $paren=$paren+1;
	} else {
	    die "Search Parenthesis Lethal Error";
	}
	if ($paren == 1) { return ($paren,1,$end_paren); }
    }
    return ($paren,0,$end_paren);
}
#|#######################################################
sub Find_Html_Parenthesis {
    local($paren,$instruction)=@_;
    local($bef_paren,$end_paren)=("","");
    unless ($instruction) {
	return ($paren,$bef_paren,$end_paren);
    }
    unless ( $instruction =~ /(\(|\))/ ) { # parenthese: complet
	$bef_paren = $instruction ;
    }
    while ( $instruction =~ /(.*?)(\(|\))(.*)/ ) { # parenthese: complet
	if ($3) {
	    $instruction = $3 ;
	} else {
	    $instruction = "";
	}
	if ($2 eq ')') {
	    $paren=$paren-1;
	} elsif ($2 eq '(') {
	    $paren=$paren+1;
	}
	$bef_paren .= $1.$2 ;
	if ($paren == 0) {
	    $end_paren = $instruction ;
	    return ($paren,$bef_paren,$end_paren);
	}
    }
    return ($paren,$bef_paren,$end_paren);
}
#|#######################################################
sub Find_Parenthesis_Old {
    local($paren,$instruction)=@_;
    local($there_remains)=1;
    local($parenmoved)=0;
    while ( $instruction =~ /(.*)\((.*)\)(.*)/ ) { # parenthese: complet
	$instruction = $1.$2.$3 ;
	$there_remains++;
	$parenmoved=1;
    }
    while ($there_remains) {
	if ( $instruction =~ /(.*)\)(.*)/ ) { # fin de parenthese
	    $instruction = $1.$2 ;
	    $paren=$paren-1;
	    $parenmoved=1;
	} elsif ( $instruction =~ /(.*)\((.*)/ ) { # debut de parenthese
	    $instruction = $1.$2 ;
	    $paren++;
	    $parenmoved=1;
	} else { # a mettre en resultat
	    $there_remains = 0 ;
	}
    }
    return ($paren,$parenmoved);
}
#%###################################################
# Enlever les commentaires du C
sub C_Instruction_Commentaire {
# Attention:
# Les variables instruction et commentaire sont globales !!!
    local($comm,$ligne)=@_;
    local($r)=$ligne;
    local($i,$c,$left,$p)=("","","","");
    $ligne="";
    while ( $r =~ /(.*?)(\/\*|\*\/|\/\/)(.*)/ ) {
	$left=$1 ; $p=$2; $r=$3;
	$ligne .= $left ;
	if ($comm) {
	    $c .= $left ;
	    if ($p eq '*/') { # fin de commentaire, dbut code
		$comm=0;
		$ligne .= &fontify::end_comment.&fontify::start_code ;
	    }
	} else {
	    $i .= $left ;
	    if ($p eq '*/') {
		die "Terminating Comment in Instruction $file $.";
	    } elsif ($p eq '/*') {
		$ligne .= &fontify::end_code.&fontify::start_comment ;
		$comm=1;
	    } elsif ($p eq '//') {
		$ligne .= &fontify::end_code.&fontify::start_comment.$r.&fontify::end_comment.&fontify::start_code ;
		$c .= $r ;
		$comm=0;
		$r="";
		last ;
	    }
	}
    }
    if ($r) {
	if ($comm) {
	    $c .= $r ;
	} else {
	    $i .= $r ;
	}
	$ligne .= $r ;
    }
    $main::instruction .= $i;
    $main::commentaire .= $c;
    return($comm,$ligne);
}
#|#######################################################
sub C_Accolade {
    local($acco,$instruction)=@_;
    local($there_remains)=1;
    local($accomoved)=0;
    while ( $instruction =~ /(.*)\{(.*)\}(.*)/ ) { # accolade: complet
	$instruction = $1.$2.$3 ;
	$there_remains++;
	$accomoved=1;
    }
    while ($there_remains) {
	if ( $instruction =~ /(.*)\}(.*)/ ) { # fin d'accolade
	    $instruction = $1.$2 ;
	    $acco=$acco-1;
	    $accomoved=1;
	} elsif ( $instruction =~ /(.*)\{(.*)/ ) { # debut d'accolade
	    $instruction = $1.$2 ;
	    $acco++;
	    $accomoved=1;
	} else { # a mettre en resultat
	    $there_remains = 0 ;
	}
    }
    return ($acco,$accomoved);
}
1;
